@@ -1,3 +1,29 @@
+##############################################################################
+## Parse variable name
+############################################################################
+
+parse.name <-function(x)
+{
+  if (is.vector(x))
+  {
+    d <- 1
+    x.names <- deparse(substitute(x))
+  }
+  else
+  {  
+    d <- ncol(x)
+    x.names <- colnames(x)
+    if (is.null(x.names))
+    {
+      x.names <- strsplit(deparse(substitute(x)), "\\[")[[1]][1]
+      x.names <- paste(x.names, "[, ", 1:d,"]",sep="") 
+    }
+  }
+  return(x.names)
+}
+
+
+
 #############################################################################
 ## Basic vectors and matrices and their operations
 #############################################################################
@@ -131,26 +157,26 @@ comm <- function(m,n){
 }
 
 ###############################################################################
-# Duplication matrix
-# Taken from Felipe Osorio http://www.ime.usp.br/~osorio/files/dupl.q
+## Duplication matrix
+## Taken from Felipe Osorio http://www.ime.usp.br/~osorio/files/dupl.q
 ###############################################################################
 
 dupl <- function(order, ret.q = FALSE)
 {
-    # call
+    ## call
     cl <- match.call()
     time1 <- proc.time()
     if (!is.integer(order))
         order <- as.integer(order)
     n <- order - 1
     
-    # initial duplication matrix
+    ## initial duplication matrix
     d1 <- matrix(0, nrow = 1, ncol = 1)
     d1[1,1] <- 1
     if (!is.integer(d1))
         storage.mode(d1) <- "integer"
     
-    # recursive formula
+    ## recursive formula
     if (n > 0){
     	for (k in 1:n){
     	    drow <- 2*k + 1 + nrow(d1)
@@ -161,9 +187,9 @@ dupl <- function(order, ret.q = FALSE)
     	    d2[2:(k+1),2:(k+1)] <- diag(k)
     	    d2[(k+2):(2*k+1),2:(k+1)] <- diag(k)
     	    d2[(2*k+2):drow,(k+2):dcol] <- d1
-    	    # permutation matrix
+    	    ## permutation matrix
     	    q <- permute.mat(k)
-    	    # new duplication matrix
+    	    ## new duplication matrix
     	    d2 <- q %*% d2
     	    storage.mode(d2) <- "integer"
     	    d1 <- d2
@@ -173,7 +199,7 @@ dupl <- function(order, ret.q = FALSE)
     	d2 <- q <- d1
     }
     
-    # results
+    ## results
     obj <- list(call=cl, order=order, d=d2)
     if (ret.q)
         obj$q <- q
@@ -181,95 +207,15 @@ dupl <- function(order, ret.q = FALSE)
     obj
 }
 
-invdupl <- function(order, ret.q = FALSE)
-{
-    # call
-    cl <- match.call()
-    time1 <- proc.time()
-    if (!is.integer(order))
-        order <- as.integer(order)
-    n <- order - 1
-
-    # initial inverse of duplication matrix
-    h1 <- matrix(0, nrow = 1, ncol = 1)
-    h1[1,1] <- 1
-
-    # recursive formula
-    if (n > 0){
-    	for (k in 1:n){
-    	    hrow <- k + 1 + nrow(h1)
-    	    hcol <- 2*k + 1 + ncol(h1)
-    	    h2 <- matrix(0, nrow = hrow, ncol=hcol)
-    	    h2[1,1] <- 1
-    	    h2[2:(k+1),2:(k+1)] <- .5*diag(k)
-    	    h2[2:(k+1),(k+2):(2*k+1)] <- .5*diag(k)
-    	    h2[(k+2):hrow,(2*k+2):hcol] <- h1
-    	    # permutation matrix
-    	    q <- permute.mat(k)
-    	    # new inverse of duplication matrix
-    	    h2 <- h2 %*% t(q)
-    	    h1 <- h2
-    	}
-    }
-    else {
-    	h2 <- q <- h1
-    }
-    
-    # results
-    obj <- list(call=cl, order=order, h=h2)
-    if (ret.q)
-        obj$q <- q
-    obj$time <- proc.time() - time1
-    obj
-}
 
 
 ###############################################################################
-# Matrix square root - taken from Stephen Lake 
-# http://www5.biostat.wustl.edu/s-news/s-news-archive/200109/msg00067.html
-###############################################################################
-
-matrix.sqrt <- function(A)
-{
-  if (length(A)==1)
-    return(sqrt(A))
-  sva <- svd(A)
-  if (min(sva$d)>=0)
-    Asqrt <- sva$u %*% diag(sqrt(sva$d)) %*% t(sva$v)
-  else
-    stop("matrix square root is not defined")
-  return(Asqrt)
-}
-
-matrix.pow <- function(A, n)
-{
-  if (nrow(A)!=ncol(A)) stop("A must be a square matrix")
-  if (floor(n)!=n) stop("n must be an integer")
-  if (n==0) return(diag(ncol(A)))
-  if (n < 0) return(matrix.pow(A=chol2inv(chol(A)), n=-n))
-        
-  # trap non-integer n and return an error
-  if (n == 1) return(A)
-  result <- diag(1, ncol(A))
-  while (n > 0) {
-    if (n %% 2 != 0) {
-      result <- result %*% A
-      n <- n - 1
-    }
-    A <- A %*% A
-    n <- n / 2
-  }
-  return(result)
-}
-
-
-###############################################################################
-# Pre-sphering
-# Parameters
-# x - data points
-#
-# Returns
-# Pre-sphered x values
+## Pre-sphering
+## Parameters
+## x - data points
+##
+## Returns
+## Pre-sphered x values
 ###############################################################################
 
 pre.sphere <- function(x, mean.centred=FALSE)
@@ -288,14 +234,50 @@ pre.sphere <- function(x, mean.centred=FALSE)
   return (x.sphered)
 }
 
+##############################################################################
+## Boolean functions
+###############################################################################
+
+is.even <- function(x)
+{
+  y <- x[x>0] %%2
+  return(identical(y, rep(0, length(y))))
+}
+
+is.diagonal <- function(x)
+{
+  return(identical(diag(diag(x)),x))
+}
 
 ###############################################################################
-# Pre-scaling
-# Parameters
-# x - data points
-#
-# Returns
-# Pre-scaled x values
+## Finds row index matrix
+## Parameters
+## x - data points
+##
+## Returns
+## i  - if r==mat[i,]
+## NA - otherwise
+###############################################################################
+
+which.mat <- function(r, mat)
+{
+  ind <- numeric()
+  
+  for (i in 1:nrow(mat))
+    if (identical(r, mat[i,])) ind <- c(ind,i)
+
+  return(ind)  
+}
+
+
+
+###############################################################################
+## Pre-scaling
+## Parameters
+## x - data points
+##
+## Returns
+## Pre-scaled x values
 ###############################################################################
 
 pre.scale <- function(x, mean.centred=FALSE)
@@ -314,32 +296,80 @@ pre.scale <- function(x, mean.centred=FALSE)
 }
 
 
-###############################################################################
-# Finds row index matrix
-# Parameters
-# x - data points
-#
-# Returns
-# i  - if r==mat[i,]
-# NA - otherwise
-###############################################################################
 
-which.mat <- function(r, mat)
+
+
+###################################################################
+## Permutation functions
+###################################################################
+
+####################################################################
+## Exactly the same function as combinat:::permn
+####################################################################
+
+permn.ks <- function (x, fun = NULL, ...) 
 {
-  ind <- numeric()
-  
-  for (i in 1:nrow(mat))
-    if (identical(r, mat[i,])) ind <- c(ind,i)
-
-  return(ind)  
+    if (is.numeric(x) && length(x) == 1 && x > 0 && trunc(x) == x) 
+      x <- seq(x)
+    n <- length(x)
+    nofun <- is.null(fun)
+    out <- vector("list", gamma(n + 1))
+    p <- ip <- seqn <- 1:n
+    d <- rep(-1, n)
+    d[1] <- 0
+    m <- n + 1
+    p <- c(m, p, m)
+    i <- 1
+    use <- -c(1, n + 2)
+    while (m != 1) {
+        out[[i]] <- if (nofun) 
+            x[p[use]]
+        else fun(x[p[use]], ...)
+        i <- i + 1
+        m <- n
+        chk <- (p[ip + d + 1] > seqn)
+        m <- max(seqn[!chk])
+        if (m < n) 
+            d[(m + 1):n] <- -d[(m + 1):n]
+        index1 <- ip[m] + 1
+        index2 <- p[index1] <- p[index1 + d[m]]
+        p[index1 + d[m]] <- m
+        tmp <- ip[index2]
+        ip[index2] <- ip[m]
+        ip[m] <- tmp
+    }
+    out
 }
 
+##########################################################################
+## Permutations with repetitions of the first d naturals (1:d) taking 
+## k elements at a time. There are d^k of them, each having length k 
+## => We arrange them into a matrix of order d^k times k
+## Each row represents one permutation
+## Second version: filling in the matrix comlumn-wise (slightly faster)
+##########################################################################
+
+perm.rep<-function(d,r)
+{
+    if(r==0){PM<-1}
+    if(r>0){
+    PM<-matrix(nrow=d^r,ncol=r)
+    for(pow in 0:(r-1)){
+        t2<-d^pow
+        p1<-1
+        while(p1<=d^r){
+            for(al in 1:d){for(p2 in 1:t2){
+                PM[p1,r-pow]<-al
+                p1<-p1+1}}}}
+    }
+    return(PM)
+}
 
 ###############################################################################
-# Permute a list of values
-#
-# Same function as EXPAND.GRID (base package), modified to take 
-# list as an argument and returns a matrix 
+## Permute a list of values
+##
+## Same function as EXPAND.GRID (base package), modified to take 
+## list as an argument and returns a matrix 
 ###############################################################################
 
 permute <- function (args) 
@@ -354,7 +384,6 @@ permute <- function (args)
                          optional = TRUE))
   cargs <- args
   rep.fac <- 1
-  ##orep <- final.len <- prod(sapply(args, length))
   orep <- prod(sapply(args, length))
   
   for (i in 1:nargs) {
@@ -382,45 +411,27 @@ permute.mat <- function(order)
     q
 }
 
-
-Theta6.elem <- function(d)
-{
-  Theta6.mat <- list()
-  Theta6.mat[[d]] <- list()
-  for (i in 1:d)
-    Theta6.mat[[i]] <- list()
-  
-  for (i in 1:d)
-    for (j in 1:d)
-    {  
-      temp <- numeric()
-      for (k in 1:d)     
-        for (ell in 1:d)    
-          temp <- rbind(temp, elem(i,d)+2*elem(k,d)+2*elem(ell,d)+elem(j,d))
-      
-      Theta6.mat[[i]][[j]] <- temp
-    }
-  
-  return(Theta6.mat)
-}
+##########################################################################
+### pinv.all generates all the permutations PR_{d,r} as described in
+### Appendix B of Chacón and Duong (2014)
+##########################################################################
+    
+pinv.all<-function(d,r){
+    i<-1:d^r
+    n<-i-1
+    dpow<-d^(0:r)
+    n.mat<-matrix(rep(n,r+1),byrow=FALSE,nrow=d^r,ncol=r+1)
+    dpow.mat<-matrix(rep(dpow,d^r),byrow=TRUE,nrow=d^r,ncol=r+1)
+    ndf.mat<-floor(n.mat/dpow.mat)
+    ans<-ndf.mat[,r:1]-d*ndf.mat[,(r+1):2]    
+    return(ans+1)
+    } 
 
 
-###############################################################################
-## Boolean functions
-###############################################################################
+##############################################################################
+## Block indices for double sums
+##############################################################################
 
-is.even <- function(x)
-{
-  y <- x[x>0] %%2
-  return(identical(y, rep(0, length(y))))
-}
-
-is.diagonal <- function(x)
-{
-  return(identical(diag(diag(x)),x))
-}
-
-## default block indices for double sums
 block.indices <- function(nx, ny, d, r=0, diff=FALSE, block.limit=1e6, npergroup)
 {
   if (missing(npergroup)) 
@@ -445,12 +456,9 @@ block.indices2 <- function(nx, ny, block.limit=1e6, npergroup)
 
 
 ####################################################################
-## Functions for unconstrained pilot selectors, and (A)MISE-optimal
-## selectors for normal mixtures
-## Author: Jose E. Chacon
+## Differences for double sums calculations
 ####################################################################
 
-
 differences <- function(x, y, upper=FALSE, ff=FALSE, Kpow=0)
 {
   if (missing(y)) y <- x
@@ -467,7 +475,7 @@ differences <- function(x, y, upper=FALSE, ff=FALSE, Kpow=0)
   for (j in 1:d)
   {
     difs[,j] <- rep(x[,j], times=ny) - rep(y[1:ny,j], each=nx)
-    ##The jth column of difs contains all the differences X_{ij}-Y_{kj}
+    ## jth column of difs contains all the differences X_{ij}-Y_{kj}
   }
  
   if (upper)
@@ -487,19 +495,96 @@ differences <- function(x, y, upper=FALSE, ff=FALSE, Kpow=0)
 
 OF<-function(m){factorial(m)/(2^(m/2)*factorial(m/2))} 
 
+###############################################################################
+## Matrix square root - taken from Stephen Lake 
+## http://www5.biostat.wustl.edu/s-news/s-news-archive/200109/msg00067.html
+###############################################################################
 
-##### Commutation matrix of order m,n
-
-K.mat<-function(m,n){
-  K<-0
-  for(i in 1:m){for(j in 1:n){
-    H<-matrix(0,nrow=m,ncol=n)
-    H[i,j]<-1
-    K<-K+(H%x%t(H))
-  }}
-  return(K)        
+matrix.sqrt <- function(A)
+{
+  if (length(A)==1)
+    return(sqrt(A))
+  sva <- svd(A)
+  if (min(sva$d)>=0)
+    Asqrt <- sva$u %*% diag(sqrt(sva$d)) %*% t(sva$v)
+  else
+    stop("matrix square root is not defined")
+  return(Asqrt)
 }
 
+###############################################################################
+## Matrix power
+###############################################################################
+
+matrix.pow <- function(A, n)
+{
+  if (nrow(A)!=ncol(A)) stop("A must be a square matrix")
+  if (floor(n)!=n) stop("n must be an integer")
+  if (n==0) return(diag(ncol(A)))
+  if (n < 0) return(matrix.pow(A=chol2inv(chol(A)), n=-n))
+        
+  # trap non-integer n and return an error
+  if (n == 1) return(A)
+  result <- diag(1, ncol(A))
+  while (n > 0) {
+    if (n %% 2 != 0) {
+      result <- result %*% A
+      n <- n - 1
+    }
+    A <- A %*% A
+    n <- n / 2
+  }
+  return(result)
+}
+
+
+##########################################################################
+### Kmat computes the commutation matrix of orders m,n
+##########################################################################    
+    
+Kmat<-function(m,n){ 
+    K<-matrix(0,ncol=m*n,nrow=m*n)
+    i<-1:m;j<-1:n
+    rows<-rowSums(expand.grid((i-1)*n,j))
+    cols<-rowSums(expand.grid(i,(j-1)*m))    
+    positions<-cbind(rows,cols)
+    K[positions]<-1
+    return(K)
+}
+
+##########################################################################
+### mat.Kprod computes row-wise Kronecker products of matrices
+########################################################################## 
+
+mat.Kprod<-function(U,V){ #### Returns a matrix with rows U[i,]%x%V[i,]
+  n1<-nrow(U)
+
+  n2<-nrow(V)
+  if(n1!=n2)stop("U and V must have the same number of vectors")
+  p<-ncol(U)
+  q<-ncol(V)
+  onep<-rep(1,p)
+  oneq<-rep(1,q)
+  P<-(U%x%t(oneq))*(t(onep)%x%V)
+  return(P)
+}
+
+
+##########################################################################
+## Kpow computes the Kronecker power of a matrix A
+##########################################################################
+
+Kpow<-function(A,pow){    
+  if(floor(pow)!=pow)stop("pow must be an integer")
+  Apow<-A
+  if(pow==0){Apow<-1}
+  if(pow>1){
+    for(i in 2:pow) Apow<-Apow%x%A
+  }
+  return(Apow)
+} 
+
+
 #### Kronecker sum
 
 Ksum <- function(A,B)
@@ -512,33 +597,6 @@ Ksum <- function(A,B)
   return(AB)
 }
 
-
-#### Kronecker power of a matrix A
-
-Kpow<-function(A,pow){   
-  if(floor(pow)!=pow)stop("pow must be an integer")
-  Apow<-A
-  if(pow==0){Apow<-1}
-  if(pow>1){
-    for(i in 2:pow) Apow<-Apow%x%A
-  }
-  return(Apow)
-}
-    
-##### Row-wise Kronecker products and powers of matrices
-    
-mat.Kprod<-function(U,V){ #### Returns a matrix with rows U[i,]%x%V[i,]
- n1<-nrow(U)
- n2<-nrow(V)
- if(n1!=n2)stop("U and V must have the same number of vectors")
- p<-ncol(U)
- q<-ncol(V)
- onep<-rep(1,p)
- oneq<-rep(1,q)
- P<-(U%x%t(oneq))*(t(onep)%x%V)
- return(P)
-} 
-
 #### Returns a matrix with the pow-th Kronecker power of A[i,] in the i-th row
 
 mat.Kpow<-function(A,pow){ 
@@ -553,206 +611,196 @@ mat.Kpow<-function(A,pow){
 
 #### Vector of all r-th partial derivatives of the normal density at x=0, i.e., D^{\otimes r)\phi(0)
 
-DrL0 <- function(d,r,Sdr.mat, Sdr.flag=TRUE, verbose=FALSE)
+DrL0 <- function(d,r)
 {
-  if (!Sdr.flag)
-  {
-    v <- Kpow(A=vec(diag(d)),pow=r/2)
-    DL0<-(-1)^(r/2)*(2*pi)^(-d/2)*OF(r)*matrix(Sdrv(d=d, r=r, v=v, verbose=verbose), ncol=1) 
-  }
-  else 
-  { 
-    if (missing(Sdr.mat)) Sdr.mat <- Sdr(d=d, r=r)
-    DL0<-(-1)^(r/2)*(2*pi)^(-d/2)*OF(r)*(Sdr.mat%*%Kpow(A=vec(diag(d)),pow=r/2))
-  }
+  v <- as.vector(Kpow(A=vec(diag(d)),pow=r/2))
+  DL0<-(-1)^(r/2)*(2*pi)^(-d/2)*OF(r)*matrix(Sdrv(d=d, r=r, v=v), ncol=1) 
   return(DL0)
 }
 
 
 
-T <- function(d,r){    #### Second version, recursive
-  Id<-diag(d)
-  Tmat<-Id
-  Kdd<-K.mat(d,d)
-  if(r>1){for(j in 2:r){
-    Idj2Kdd<-diag(d^(j-2))%x%Kdd
-    Tmat<-Idj2Kdd%*%((Tmat%x%Id)+Idj2Kdd)%*%Idj2Kdd
-  }}
-  return(Tmat)
+#########################################################################
+### Wrapper functions for Chacon & Duong (2014) 
+##########################################################################
+
+Sdr<-function(d, r, type="recursive"){
+  type1 <- match.arg(type, c("recursive", "direct"))
+  Sdr.mat <- do.call(paste("Sdr", type1, sep="."), list(d=d, r=r))
+  return(Sdr.mat)
 }
 
-T2 <- function(d,r){    #### Second version, recursive
-  Tmat<-diag(d^r)
-  if(r>1){
-  for(j in 1:(r-1)){
-    Tmat<-Tmat+(diag(d^j)%x%K.mat(d^(r-j-1),d))%*%(diag(d^(j-1))%x%K.mat(d,d^(r-j)))
-    }}
-  return(Tmat)
-}
-
-Trow <- function(d,r,row=1){    #### Second version, recursive
-  Id <- diag(d)
-  Tmat <- Id
-  Kdd <- K.mat(d,d)
-  if (r>1)
-  {
-    for (j in 2:r)
-    {
-     Idj2Kdd <- diag(d^(j-2)) %x% Kdd
-     Tmat <- Idj2Kdd %*% ((Tmat %x% Id) + Idj2Kdd) %*% Idj2Kdd
-   }
-  } 
-  return(Tmat)
+Sdrv<-function(d, r, v, type="recursive"){
+  type1 <- match.arg(type, c("recursive", "direct"))
+  v <- as.vector(v)
+  Sdrvec <- do.call(paste("Sdrv", type1, sep="."), list(d=d, r=r, v=v))
+  return(Sdrvec)
 }
 
 
-
-## symmetriser matrix
-
-Sdr<-function(d, r){
-  if (r==0) S<-1
-  else{
-    S <- diag(d)
-    if (r>=2)
-      for(j in 2:r)
-        S <- S %x% diag(d) %*% T(d,j)/j
-  }
-  return(S)
-}
-
-#### Permutations with repetitions of the first d naturals (1:d) taking 
-#### k elements at a time. There are d^k of them, each having length k 
-#### => We arrange them into a matrix of order d^k times k
-#### Each row represents one permutation
-#### Second version: filling in the matrix comlumn-wise (slightly faster)
-    
-perm.rep<-function(d,r)
-{
-    if(r==0){PM<-1}
-    if(r>0){
-    PM<-matrix(nrow=d^r,ncol=r)
-    for(pow in 0:(r-1)){
-        t2<-d^pow
-        p1<-1
-        while(p1<=d^r){
-            for(al in 1:d){for(p2 in 1:t2){
-                PM[p1,r-pow]<-al
-                p1<-p1+1}}}}
-    }
-    return(PM)
-}
+##########################################################################
+## Symmetriser matrix
+##########################################################################
 
 
-### Symmetriser applied to a vector
+############################################################################
+### Sdr.direct computes the symmetrizer matrix S_{d,r} based on Equation (4)
+### as described in Section 3 of Chacón and Duong (2014)
+############################################################################
 
-Sdrv.old <- function(d,r,v, verbose=FALSE)
-{   
-    if(length(v)!=d^r){stop("length of v must equal d^r")}
-    per.rep<-perm.rep(d,r)
+Sdr.direct<-function(d,r){ 
+    S<-matrix(0,ncol=d^r,nrow=d^r)
+    per<-permn.ks(r)
+    per.rep<-pinv.all(d,r)
     nper<-factorial(r)
-    ##nper.rep<-d^r
-    dpow<-d^((r-1):0)
-    dpow.mat<-matrix(rep(dpow,d^r),ncol=r,nrow=d^r,byrow=TRUE)    
-    if (verbose) pb <- txtProgressBar()
+    nper.rep<-d^r
+    per<-matrix(unlist(per), byrow=TRUE, ncol=r, nrow=nper)    
+    pow<-0:(r-1)
+    dpow<-d^pow
     
-    ## modified from the permn function in the combinat library 02/2012   
-  
-    Sv <- rep(0,d^r)
-    x <- seq(r)
-    n <- length(x)
-    ##nofun <- is.null(fun)
-    ##out <- vector("list", gamma(n + 1))
-    p <- ip <- seqn <- 1:n
-    d <- rep(-1, n)
-    d[1] <- 0
-    m <- n + 1
-    p <- c(m, p, m)
-    i <- 1
- 
-    nper.thin <- 0
-    use <- -c(1, n + 2)
+    if(nper.rep<=nper){
+    dpow.mat<-matrix(rep(dpow,nper),byrow=TRUE,ncol=r,nrow=nper)
+    for(i in 1:nper.rep){      ## Loop over no. perms with reps (d^r)
+        pinvi<-per.rep[i,]
+        sigpinvi<-matrix(pinvi[per],byrow=FALSE,nrow=nrow(per),ncol=ncol(per))
+        psigpinvi<-drop(1+rowSums((sigpinvi-1)*dpow.mat))
+        S[i,]<-tabulate(psigpinvi,nbins=d^r)
+    }}
+    
+    if(nper<nper.rep){
+    dpow.mat<-matrix(rep(dpow,nper.rep),byrow=TRUE,ncol=r,nrow=nper.rep)
+    for(s in 1:nper){          ## Loop over no. perms (r!)
+        sig<-per[s,]
+        sigpinv<-per.rep[,sig]
+        psigpinv<-drop(1+rowSums((sigpinv-1)*dpow.mat))
+        locations<-cbind(1:d^r,psigpinv)
+        S[locations]<-S[locations]+1
+    }}    
+    return(S/nper)
+}
+     
+############################################################################
+### Sdr.recursive computes the symmetrizer matrix S_{d,r} based on 
+### the recursive approach detailed in Algorithm 1 in Section 3 of 
+### Chacón and Duong (2014)
+############################################################################     
+     
+Sdr.recursive<-function(d,r){
+    S<-diag(d)
+    if(r==0)S<-1
+    if(r>=2){
+        Id<-diag(d)
+        T<-Id 
+        A<-Kmat(d,d)        
+        for(j in 2:r){
+            T<-((j-1)/j)*(A%*%(T%x%Id)%*%A)+A/j
+            S<-(S%x%Id)%*%T
+            if(j<r){A<-Id%x%A}
+            }}
+    return(S)
+}
+
+
+
+##########################################################################
+### Symmetrizer matrix applied to a vector
+##########################################################################
+    
+
+############################################################################
+### Sdrv.direct computes the result of multiplying the symmetrizer matrix
+### S_{d,r} by a vector v of length d^r, based on Equation (4) as described
+### in Section 4 of Chacón and Duong (2014)
+############################################################################ 
+
+Sdrv.direct<-function(d,r,v){
+    Sv<-rep(0,d^r)
+    per<-permn.ks(r)
+    per.rep<-pinv.all(d,r)
+    nper<-factorial(r)
+    nper.rep<-d^r
+    per<-matrix(unlist(per), byrow=TRUE, ncol=r, nrow=nper)    
+    pow<-0:(r-1)
+    dpow<-d^pow
+    
+    if(nper.rep<=nper){
+    dpow.mat<-matrix(rep(dpow,nper),byrow=TRUE,ncol=r,nrow=nper)
+    for(i in 1:nper.rep){      ## Loop over no. perms with reps (d^r)
+        pinvi<-per.rep[i,]
+        sigpinvi<-matrix(pinvi[per],byrow=FALSE,nrow=nrow(per),ncol=ncol(per))
+        psigpinvi<-drop(1+rowSums((sigpinvi-1)*dpow.mat))
+        Sv[i]<-sum(tabulate(psigpinvi,nbins=d^r)*v)
+    }}
+    
+    if(nper<nper.rep){
+    dpow.mat<-matrix(rep(dpow,nper.rep),byrow=TRUE,ncol=r,nrow=nper.rep)
+    for(s in 1:nper){          ## Loop over no. perms (r!)
+        sig<-per[s,]
+        sigpinv<-per.rep[,sig]
+        psigpinv<-drop(1+rowSums((sigpinv-1)*dpow.mat))
+        Sv<-Sv+v[psigpinv]
+    }}    
+    return(Sv/nper)
+}
    
-    while (m != 1) {
-      ##out <- if (nofun) x[p[use]] else fun(x[p[use]], ...)
-      out <- x[p[use]]
-       
-      ## next 2 lines added to compute Sdr%*%v product
-      if (verbose) setTxtProgressBar(pb, i/nper)
-      ##cat(paste(i,"/", nper, "\n", sep=""))
-      Sv <- Sv + v[1+rowSums((per.rep[,out]-1)*dpow.mat)]
-      nper.thin <- nper.thin+1
-      
-      i <- i + 1
-      m <- n
-      chk <- (p[ip + d + 1] > seqn)
-      m <- max(seqn[!chk])
-      if (m < n) d[(m + 1):n] <- -d[(m + 1):n]
-      index1 <- ip[m] + 1
-      index2 <- p[index1] <- p[index1 + d[m]]
-      p[index1 + d[m]] <- m
-      tmp <- ip[index2]
-      ip[index2] <- ip[m]
-      ip[m] <- tmp
-   }
-   permnr <- Sv/nper.thin  
-   if (verbose) close(pb)
-   return(permnr)
-}
 
+############################################################################
+### Sdrv.recursive computes the result of multiplying the symmetrizer matrix
+### S_{d,r} by a vector v of length d^r, based on the recursive Algorithm 2
+### as described in Section 4 of Chacón and Duong (2014)
+############################################################################ 
 
-Sdrv <- function(d,r,v, verbose=FALSE)
-{
-    if(length(v)!=d^r){stop("length of v must equal d^r")}
-    if(r==0){w<-v}
+Sdrv.recursive<-function(d,r,v){
+    if((!is.vector(v))&(!is.matrix(v))){stop("v must be a vector or a matrix")}
+    if(is.vector(v)){v<-matrix(v,nrow=1)}
+    n<-nrow(v)   
+    if(ncol(v)!=d^r){stop("The length of the vector(s) must equal d^r")}
+    
+    if((r==0)|(r==1)){w<-v}
+    
     else{
-    per.rep <- perm.rep(d,r)
-    ##nper.rep <- d^r
-    dpow <- d^((r-1):0)
-    dpow.mat <- matrix(rep(dpow,d^r),ncol=r,nrow=d^r,byrow=TRUE) 
+    per.rep<-pinv.all(d,r)
+    nper.rep<-d^r
+    dpow<-d^((r-1):0)
+    dpow.mat<-matrix(rep(dpow,d^r),ncol=r,nrow=d^r,byrow=TRUE) 
 
-    i <- 0
-    if (verbose) pb <- txtProgressBar()
-    w <- v
-    for(p in 1:r){
-        Tv <- rep(0,d^r)
-        for (j in 1:p){
-        i <- i+1
-        if (verbose) setTxtProgressBar(pb, i/(r*(r+1)/2))
-        per.rep.tau <- per.rep
-        per.rep.tau[,j] <- per.rep[,p]
-        per.rep.tau[,p] <- per.rep[,j]
-        Tv[1+rowSums((per.rep.tau-1)*dpow.mat)] <- Tv[1+rowSums((per.rep.tau-1)*dpow.mat)]+w
+    w<-v
+    for(p in 2:r){
+        Tv<-matrix(0,nrow=n,ncol=d^r)
+        for(j in 1:p){
+        per.rep.tau<-per.rep;per.rep.tau[,j]<-per.rep[,p];per.rep.tau[,p]<-per.rep[,j]
+        positions<-1+rowSums((per.rep.tau-1)*dpow.mat)
+        Tv[,positions]<-Tv[,positions]+w
     }
-    w <- Tv/p}
+    w<-Tv/p}
     }
-    if (verbose) close (pb)
-    return(w)
+    return(drop(w))
 }
 
 
 
+
 #############################################################################
-## Basic operations
+## Lp norm between two functions (grid based)
 #############################################################################
 
-## Parse variable name
-parse.name <-function(x)
+Lpdiff <- function(f1, f2, p=2)
 {
-  if (is.vector(x))
+  d <- ncol(f1$H)
+  f.diff <- f1
+  f.diff$estimate <- (abs(f1$estimate - f2$estimate))^p
+  if (d==1)
   {
-    d <- 1
-    x.names <- deparse(substitute(x))
+    delta <- diff(f.diff$eval.points)
+    riemann.sum <- sum(c(delta[1], delta)*f.diff$estimate)
   }
-  else
-  {  
-    d <- ncol(x)
-    x.names <- colnames(x)
-    if (is.null(x.names))
-    {
-      x.names <- strsplit(deparse(substitute(x)), "\\[")[[1]][1]
-      x.names <- paste(x.names, "[, ", 1:d,"]",sep="") 
-    }
-  }
-  return(x.names)
+  else if (d>1)
+  {
+    delta <- sapply(f.diff$eval.points, diff)
+    delta <- rbind(head(delta, n=1), delta)
+    if (d==2) riemann.sum <- sum(outer(delta[,1], delta[,2]) * f.diff$estimate)
+    else if (d==3) riemann.sum <- sum(outer(outer(delta[,1], delta[,2]), delta[,3]) * f.diff$estimate)
+  }  
+  
+  return(riemann.sum)
 }
-
